@@ -2,20 +2,28 @@
 import os
 import time
 import json
+import yaml
 import pickle
+import datetime
+import subprocess
 import collections.abc
 from glob import glob
 from functools import wraps
 
 # Libs
 import numpy as np
+import pandas as pd
 from PIL import Image
+from tqdm import tqdm
 from skimage import io
 from natsort import natsorted
 
 # Own modules
 
 
+# TODO add test modules
+
+
 def make_dir_if_not_exist(dir_path):
     """
     Make the directory if it does not exists
@@ -71,15 +79,20 @@ def load_file(file_name, **kwargs):
             with open(file_name, 'r') as f:
                 data = f.readlines()
         elif file_name[-3:] == 'csv':
-            data = np.genfromtxt(file_name, **kwargs)
+            data = pd.read_csv(file_name, **kwargs)
         elif file_name[-4:] == 'json':
             data = json.load(open(file_name))
+        elif file_name[-4:] == 'yaml':
+            with open(file_name, 'r') as stream:
+                data = yaml.safe_load(stream)
         elif 'pil' in kwargs and kwargs['pil']:
             try:
                 data = Image.open(file_name)
             except Image.DecompressionBombError:
                 Image.MAX_IMAGE_PIXELS = None
                 data = Image.open(file_name)
+            if 'to_numpy' in kwargs and kwargs['to_numpy']:
+                data = np.array(data)
         else:
             try:
                 data = io.imread(file_name)
@@ -87,13 +100,30 @@ def load_file(file_name, **kwargs):
                 Image.MAX_IMAGE_PIXELS = None
                 data = io.imread(file_name)
             except ValueError or OSError:
-                data = np.array(Image.open(file_name).convert('RGB'))
+                data = np.array(Image.open(file_name))
 
         return data
     except Exception:  # so many things could go wrong, can't be more specific.
         raise IOError('Problem loading {}'.format(file_name))
 
 
+def load_files(file_name, **kwargs):
+    """
+    Load a list of files
+    :param file_name: could be either the path of one file or list of files
+    :return: file data, or IOError if it cannot be read by either numpy or pickle or imageio
+    """
+    if isinstance(file_name, str):
+        return load_file(file_name, **kwargs)
+    elif isinstance(file_name, list) or isinstance(file_name, str):
+        data = []
+        for f in file_name:
+            data.append(load_file(f, **kwargs))
+        return data
+    else:
+        raise TypeError('file_name type {} not understood'.format(type(file_name)))
+
+
 def save_file(file_name, data, fmt='%.8e', sort_keys=True, indent=4):
     """
     Save data file of given path, use numpy.load if it is in .npy format,
@@ -122,6 +152,23 @@ def save_file(file_name, data, fmt='%.8e', sort_keys=True, indent=4):
         raise IOError('Problem saving this data')
 
 
+def save_files(file_name, data, fmt='%.8e', sort_keys=True, indent=4):
+    """
+    Save a list of files
+    :param file_name: could be either the path of one file or list of files
+    :return: file data, or IOError if it cannot be read by either numpy or pickle or imageio
+    """
+    if isinstance(file_name, str):
+        return save_file(file_name, data, fmt, sort_keys, indent)
+    elif isinstance(file_name, list) or isinstance(file_name, str):
+        data = []
+        for f in file_name:
+            data.append(save_file(f, data, fmt, sort_keys, indent))
+        return data
+    else:
+        raise TypeError('file_name type {} not understood'.format(type(file_name)))
+
+
 def rotate_list(l):
     """
     Rotate a list of lists
@@ -272,25 +319,6 @@ def parse_args(arg_list):
     return arg_dict
 
 
-def change_channel_order(data, to_channel_last=True):
-    """
-    Switch the image type from channel first to channel last
-    :param data: the data to switch the channels
-    :param to_channel_last: if True, switch the first channel to the last
-    :return: the channel switched data
-    """
-    if to_channel_last:
-        if len(data.shape) == 3:
-            return np.rollaxis(data, 0, 3)
-        else:
-            return np.rollaxis(data, 1, 4)
-    else:
-        if len(data.shape) == 3:
-            return np.rollaxis(data, 2, 0)
-        else:
-            return np.rollaxis(data, 3, 1)
-
-
 def verb_print(txt, verbose=True):
     """
     Only print the txt if verbose is set to True
@@ -300,3 +328,63 @@ def verb_print(txt, verbose=True):
     """
     if verbose:
         print(txt)
+
+
+def get_file_length(file_name):
+    """
+    Get the file length, this would be useful to know the total length for tqdm when processing large file
+    :param file_name: the path to the file
+    :return:
+    """
+    return int(subprocess.check_output('wc -l {}'.format(file_name), shell=True).split()[0])
+
+
+def get_time_str(time_fmt='%Y-%m-%d_%H-%M-%S'):
+    """
+    Return the formatted time string, the default pattern has no backslash so it can be used in directory path
+    :param time_fmt:
+    :return:
+    """
+    return datetime.datetime.now().strftime(time_fmt)
+
+
+def random_split(array, portions, seed=None):
+    """
+    Randomly split an array into given portions
+    :param array: an iterable object, will be splitted into different portions
+    :param portions: a list of portions, should be summed to 1, otherwise the last element will be the rest elements
+    :param seed: if given, the seed will be set, this is for reproducibility
+    :return:
+    """
+    if seed is not None:
+        np.random.seed(seed)
+    if np.sum(portions) < 1 and np.sum(portions) > 0:
+        portions = list(portions)
+        portions.append(1 - np.sum(portions))
+    elif np.sum(portions) > 1 or np.sum(portions) < 0:
+        raise ValueError
+
+    array_len = len(array)
+    rand_idx = np.random.permutation(np.arange(array_len))
+    portions = [int(np.floor(a * array_len)) for a in portions]
+    for cnt in range(1, len(portions)):
+        portions[cnt] = portions[cnt-1] + portions[cnt]
+
+    splits = [[array[a] for a in rand_idx[:portions[0]]]]
+    for lb, rb in zip(portions[:-1], portions[1:]):
+        splits.append([array[a] for a in rand_idx[lb:rb]])
+
+    return splits
+
+
+def create_symlinks(source_files, dest_dir):
+    """
+    Create symbolic links of certain files in another directory
+    :param source_files: absolute path of the files
+    :param dest_dir: destination of the folder to have the symbolic links
+
+    :return:
+    """
+    for source_file in tqdm(source_files):
+        target_file = os.path.join(dest_dir, os.path.basename(source_file))
+        os.symlink(source_file, target_file)
